# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import unittest
import time

import calendar
from datetime import datetime, timezone
from azure.core.credentials import AccessToken
from azure.core.exceptions import HttpResponseError
from azure.communication.chat import ChatThreadClient, ChatParticipant, ChatMessageType
from azure.communication.chat._shared.models import CommunicationUserIdentifier
from unittest_helpers import mock_response

from unittest.mock import Mock, patch


def _convert_datetime_to_utc_int(input):
    return int(calendar.timegm(input.utctimetuple()))


class TestChatThreadClient(unittest.TestCase):
    @classmethod
    @patch("azure.communication.identity._shared.user_credential.CommunicationTokenCredential")
    def setUpClass(cls, credential):
        credential.get_token = Mock(
            return_value=AccessToken(
                "some_token", _convert_datetime_to_utc_int(datetime.now().replace(tzinfo=timezone.utc))
            )
        )
        TestChatThreadClient.credential = credential

    def test_update_topic(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=204)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        topic = "update topic"
        try:
            chat_thread_client.update_topic(topic=topic)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_send_message(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=201, json_payload={"id": message_id})

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        create_message_result = None
        try:
            content = "hello world"
            sender_display_name = "sender name"
            metadata = {"tags": "tag"}
            create_message_result = chat_thread_client.send_message(
                content=content, sender_display_name=sender_display_name, metadata=metadata
            )
            create_message_result_id = create_message_result.id
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        assert create_message_result_id == message_id

    def test_send_message_w_type(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False
        message_str = "Hi I am Bob."

        chat_message_types = [ChatMessageType.TEXT, ChatMessageType.HTML, "text", "html"]

        for chat_message_type in chat_message_types:

            def mock_send(*_, **__):
                return mock_response(
                    status_code=201,
                    json_payload={
                        "id": message_id,
                        "type": chat_message_type,
                        "sequenceId": "3",
                        "version": message_id,
                        "content": {
                            "message": message_str,
                            "topic": "Lunch Chat thread",
                            "participants": [
                                {
                                    "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b",
                                    "displayName": "Bob",
                                    "shareHistoryTime": "2020-10-30T10:50:50Z",
                                }
                            ],
                            "initiator": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b",
                        },
                        "senderDisplayName": "Bob",
                        "createdOn": "2021-01-27T01:37:33Z",
                        "senderId": "8:acs:46849534-eb08-4ab7-bde7-c36928cd1547_00000007-e155-1f06-1db7-3a3a0d00004b",
                    },
                )

            chat_thread_client = ChatThreadClient(
                "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
            )

            try:
                content = "hello world"
                sender_display_name = "sender name"
                create_message_result = chat_thread_client.send_message(
                    content=content, chat_message_type=chat_message_type, sender_display_name=sender_display_name
                )
                create_message_result_id = create_message_result.id
            except:
                raised = True

            self.assertFalse(raised, "Expected is no excpetion raised")
            assert create_message_result_id == message_id

    def test_send_message_w_invalid_type_throws_error(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False
        message_str = "Hi I am Bob."

        # the payload is irrelevant - it'll fail before
        def mock_send(*_, **__):
            return mock_response(status_code=201, json_payload={"id": message_id})

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        create_message_result = None

        chat_message_types = [
            ChatMessageType.PARTICIPANT_ADDED,
            ChatMessageType.PARTICIPANT_REMOVED,
            ChatMessageType.TOPIC_UPDATED,
            "participant_added",
            "participant_removed",
            "topic_updated",
            "ChatMessageType.TEXT",
            "ChatMessageType.HTML",
            "ChatMessageType.PARTICIPANT_ADDED",
            "ChatMessageType.PARTICIPANT_REMOVED",
            "ChatMessageType.TOPIC_UPDATED",
        ]
        for chat_message_type in chat_message_types:
            try:
                content = "hello world"
                sender_display_name = "sender name"
                create_message_result = chat_thread_client.send_message(
                    content=content, chat_message_type=chat_message_type, sender_display_name=sender_display_name
                )
            except:
                raised = True

            self.assertTrue(raised, "Expected is excpetion raised")

    def test_get_message(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False
        message_str = "Hi I am Bob."

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "id": message_id,
                    "type": "text",
                    "sequenceId": "3",
                    "version": message_id,
                    "content": {
                        "message": message_str,
                        "topic": "Lunch Chat thread",
                        "participants": [
                            {
                                "communicationIdentifier": {
                                    "rawId": "string",
                                    "communicationUser": {
                                        "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                    },
                                },
                                "displayName": "Bob",
                                "shareHistoryTime": "2020-10-30T10:50:50Z",
                            }
                        ],
                        "initiatorCommunicationIdentifier": {
                            "rawId": "string",
                            "communicationUser": {
                                "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                            },
                        },
                        "attachments": [
                            {
                                "id": "id",
                                "attachmentType": "image",
                                "name": "name.png",
                                "url": "https://endpoint/threads/chatThreadId/images/imageId/views/original",
                                "previewUrl": "https://endpoint/threads/chatThreadId/images/imageId/views/preview",
                            },
                            {
                                "id": "id",
                                "attachmentType": "file",
                                "name": "name.pdf",
                                "previewUrl": "https://contoso.sharepoint.com/teams/TeamName/DocumentLibrary/FileName",
                            },
                        ],
                    },
                    "senderDisplayName": "Bob",
                    "createdOn": "2021-01-27T01:37:33Z",
                    "senderCommunicationIdentifier": {
                        "rawId": "string",
                        "communicationUser": {
                            "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                        },
                    },
                    "deletedOn": "2021-01-27T01:37:33Z",
                    "editedOn": "2021-01-27T01:37:33Z",
                    "metadata": {"tags": "tag"},
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        message = None
        try:
            message = chat_thread_client.get_message(message_id)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        assert message.id == message_id
        assert message.content.message == message_str
        assert message.type == ChatMessageType.TEXT
        assert message.metadata["tags"] == "tag"
        assert len(message.content.participants) > 0
        assert len(message.content.attachments) == 2
        assert message.content.attachments[0].attachment_type == "image"
        assert message.content.attachments[1].attachment_type == "file"

    def test_list_messages(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        message_str = "Hi I am Bob."
        raised = False

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "value": [
                        {
                            "id": message_id,
                            "type": "text",
                            "sequenceId": "3",
                            "version": message_id,
                            "content": {
                                "message": message_str,
                                "topic": "Lunch Chat thread",
                                "participants": [
                                    {
                                        "communicationIdentifier": {
                                            "rawId": "string",
                                            "communicationUser": {
                                                "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                            },
                                        },
                                        "displayName": "Bob",
                                        "shareHistoryTime": "2020-10-30T10:50:50Z",
                                    }
                                ],
                                "initiatorCommunicationIdentifier": {
                                    "rawId": "string",
                                    "communicationUser": {
                                        "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                    },
                                },
                            },
                            "senderDisplayName": "Bob",
                            "createdOn": "2021-01-27T01:37:33Z",
                            "senderCommunicationIdentifier": {
                                "rawId": "string",
                                "communicationUser": {
                                    "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                },
                            },
                            "deletedOn": "2021-01-27T01:37:33Z",
                            "editedOn": "2021-01-27T01:37:33Z",
                        }
                    ]
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        chat_messages = None
        try:
            chat_messages = chat_thread_client.list_messages(results_per_page=1)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        for chat_message in chat_messages.by_page():
            l = list(chat_message)
            assert len(l) == 1
            assert l[0].id == message_id

    def test_list_messages_with_start_time(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        raised = False
        message_id = "1596823919339"
        message_str = "Hi I am Bob."

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "value": [
                        {
                            "id": message_id,
                            "type": "text",
                            "sequenceId": "2",
                            "version": message_id,
                            "content": {
                                "message": message_str,
                                "topic": "Lunch Chat thread",
                                "participants": [
                                    {
                                        "communicationIdentifier": {
                                            "rawId": "string",
                                            "communicationUser": {
                                                "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                            },
                                        },
                                        "displayName": "Bob",
                                        "shareHistoryTime": "2020-10-30T10:50:50Z",
                                    }
                                ],
                                "initiatorCommunicationIdentifier": {
                                    "rawId": "string",
                                    "communicationUser": {
                                        "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                    },
                                },
                            },
                            "senderDisplayName": "Bob",
                            "createdOn": "2021-01-27T01:37:33Z",
                            "senderCommunicationIdentifier": {
                                "rawId": "string",
                                "communicationUser": {
                                    "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                },
                            },
                            "deletedOn": "2021-01-27T01:37:33Z",
                            "editedOn": "2021-01-27T01:37:33Z",
                        },
                        {
                            "id": message_id,
                            "type": "text",
                            "sequenceId": "3",
                            "version": message_id,
                            "content": {
                                "message": message_str,
                                "topic": "Lunch Chat thread",
                                "participants": [
                                    {
                                        "communicationIdentifier": {
                                            "rawId": "string",
                                            "communicationUser": {
                                                "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                            },
                                        },
                                        "displayName": "Bob",
                                        "shareHistoryTime": "2020-10-30T10:50:50Z",
                                    }
                                ],
                                "initiatorCommunicationIdentifier": {
                                    "rawId": "string",
                                    "communicationUser": {
                                        "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                    },
                                },
                            },
                            "senderDisplayName": "Bob",
                            "createdOn": "2021-01-27T01:37:33Z",
                            "senderCommunicationIdentifier": {
                                "rawId": "string",
                                "communicationUser": {
                                    "id": "8:acs:8540c0de-899f-5cce-acb5-3ec493af3800_0e59221d-0c1d-46ae-9544-c963ce56c10b"
                                },
                            },
                            "deletedOn": "2021-01-27T01:37:33Z",
                            "editedOn": "2021-01-27T01:37:33Z",
                        },
                    ]
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        chat_messages = None
        try:
            chat_messages = chat_thread_client.list_messages(start_time=datetime(2020, 8, 17, 18, 0, 0))
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        for chat_message in chat_messages.by_page():
            l = list(chat_message)
            assert len(l) == 2

    def test_update_message_content(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=204)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        try:
            content = "updated message content"
            chat_thread_client.update_message(message_id, content=content)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_update_message_metadata(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=204)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        try:
            metadata = {"tags": "tag"}
            chat_thread_client.update_message(message_id, metadata=metadata)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_delete_message(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=204)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        try:
            chat_thread_client.delete_message(message_id)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_list_participants(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        participant_id = "8:acs:57b9bac9-df6c-4d39-a73b-26e944adf6ea_9b0110-08007f1041"
        raised = False

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "value": [
                        {
                            "communicationIdentifier": {
                                "rawId": participant_id,
                                "communicationUser": {"id": participant_id},
                            },
                            "displayName": "Bob",
                            "shareHistoryTime": "2020-10-30T10:50:50Z",
                        }
                    ]
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        chat_thread_participants = None
        try:
            chat_thread_participants = chat_thread_client.list_participants()
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        for chat_thread_participant_page in chat_thread_participants.by_page():
            l = list(chat_thread_participant_page)
            assert len(l) == 1
            l[0].identifier.properties["id"] = participant_id

    def test_list_participants_with_results_per_page(self):
        thread_id = "19:81181a8abbf54b5695f87a0042ddcba9@thread.v2"
        participant_id_1 = "8:acs:9b665d53-8164-4923-ad5d-5e983b07d2e7_00000006-5399-552c-b274-5a3a0d0000dc"
        participant_id_2 = "8:acs:9b665d53-8164-4923-ad5d-5e983b07d2e7_00000006-9d32-35c9-557d-5a3a0d0002f1"
        raised = False

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "value": [
                        {
                            "communicationIdentifier": {
                                "rawId": participant_id_1,
                                "communicationUser": {"id": participant_id_1},
                            },
                            "displayName": "Bob",
                            "shareHistoryTime": "2020-10-30T10:50:50Z",
                        },
                        {
                            "communicationIdentifier": {
                                "rawId": participant_id_2,
                                "communicationUser": {"id": participant_id_2},
                            },
                            "displayName": "Bob",
                            "shareHistoryTime": "2020-10-30T10:50:50Z",
                        },
                    ]
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        chat_thread_participants = None
        try:
            chat_thread_participants = chat_thread_client.list_participants(results_per_page=2)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        for chat_thread_participant_page in chat_thread_participants.by_page():
            l = list(chat_thread_participant_page)
            assert len(l) == 2

    def test_add_participants(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        new_participant_id = "8:acs:57b9bac9-df6c-4d39-a73b-26e944adf6ea_9b0110-08007f1041"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=201)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        new_participant = ChatParticipant(
            identifier=CommunicationUserIdentifier(new_participant_id),
            display_name="name",
            share_history_time=datetime.utcnow(),
        )
        participants = [new_participant]

        try:
            result = chat_thread_client.add_participants(participants)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        self.assertTrue(len(result) == 0)

    def test_add_participants_w_failed_participants_returns_nonempty_list(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        new_participant_id = "8:acs:57b9bac9-df6c-4d39-a73b-26e944adf6ea_9b0110-08007f1041"
        raised = False
        error_message = "some error message"

        def mock_send(*_, **__):
            return mock_response(
                status_code=201,
                json_payload={
                    "invalidParticipants": [
                        {"code": "string", "message": error_message, "target": new_participant_id, "details": []}
                    ]
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        new_participant = ChatParticipant(
            identifier=CommunicationUserIdentifier(new_participant_id),
            display_name="name",
            share_history_time=datetime.utcnow(),
        )
        participants = [new_participant]

        try:
            result = chat_thread_client.add_participants(participants)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        self.assertTrue(len(result) == 1)

        failed_participant = result[0][0]
        communication_error = result[0][1]

        self.assertEqual(new_participant.identifier.properties["id"], failed_participant.identifier.properties["id"])
        self.assertEqual(new_participant.display_name, failed_participant.display_name)
        self.assertEqual(new_participant.share_history_time, failed_participant.share_history_time)
        self.assertEqual(error_message, communication_error.message)

    def test_remove_participant(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        participant_id = "8:acs:57b9bac9-df6c-4d39-a73b-26e944adf6ea_9b0110-08007f1041"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=204)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        try:
            chat_thread_client.remove_participant(identifier=CommunicationUserIdentifier(participant_id))
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_send_typing_notification(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=200)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        try:
            chat_thread_client.send_typing_notification()
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_send_typing_notification_with_sender_display_name(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=200)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        try:
            chat_thread_client.send_typing_notification(sender_display_name="John")
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_send_read_receipt(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False

        def mock_send(*_, **__):
            return mock_response(status_code=200)

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        try:
            chat_thread_client.send_read_receipt(message_id)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")

    def test_list_read_receipts(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id = "1596823919339"
        raised = False

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "value": [
                        {
                            "chatMessageId": message_id,
                            "senderCommunicationIdentifier": {"rawId": "string", "communicationUser": {"id": "string"}},
                        }
                    ]
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        read_receipts = None
        try:
            read_receipts = chat_thread_client.list_read_receipts()
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        for read_receipt_page in read_receipts.by_page():
            l = list(read_receipt_page)
            assert len(l) == 1

    def test_list_read_receipts_with_results_per_page(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        message_id_1 = "1596823919339"
        message_id_2 = "1596823919340"
        raised = False

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "value": [
                        {
                            "chatMessageId": message_id_1,
                            "senderCommunicationIdentifier": {"rawId": "string", "communicationUser": {"id": "string"}},
                        },
                        {
                            "chatMessageId": message_id_2,
                            "senderCommunicationIdentifier": {"rawId": "string", "communicationUser": {"id": "string"}},
                        },
                    ]
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        read_receipts = None
        try:
            read_receipts = chat_thread_client.list_read_receipts(results_per_page=2)
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        for read_receipt_page in read_receipts.by_page():
            l = list(read_receipt_page)
            assert len(l) == 2

    def test_get_properties(self):
        thread_id = "19:bcaebfba0d314c2aa3e920d38fa3df08@thread.v2"
        raised = False

        def mock_send(*_, **__):
            return mock_response(
                status_code=200,
                json_payload={
                    "id": thread_id,
                    "topic": "Lunch Chat thread",
                    "createdOn": "2020-10-30T10:50:50Z",
                    "deletedOn": "2020-10-30T10:50:50Z",
                    "createdByCommunicationIdentifier": {"rawId": "string", "communicationUser": {"id": "string"}},
                },
            )

        chat_thread_client = ChatThreadClient(
            "https://endpoint", TestChatThreadClient.credential, thread_id, transport=Mock(send=mock_send)
        )

        get_thread_result = None
        try:
            get_thread_result = chat_thread_client.get_properties()
        except:
            raised = True

        self.assertFalse(raised, "Expected is no excpetion raised")
        assert get_thread_result.id == thread_id


if __name__ == "__main__":
    unittest.main()



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to use agent operations with an event handler in streaming from
    the Azure Agents service using a synchronous client.

USAGE:
    python sample_agents_stream_eventhandler.py

    Before running the sample:

    pip install azure-ai-agents azure-identity

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Azure AI Foundry project.
"""

import os
from azure.ai.agents import AgentsClient
from azure.identity import DefaultAzureCredential

from azure.ai.agents.models import (
    AgentEventHandler,
    MessageDeltaChunk,
    ThreadMessage,
    ThreadRun,
    RunStep,
)

from typing import Any, Optional

agents_client = AgentsClient(
    endpoint=os.environ["PROJECT_ENDPOINT"],
    credential=DefaultAzureCredential(),
)


# [START stream_event_handler]
# With AgentEventHandler[str], the return type for each event functions is optional string.
class MyEventHandler(AgentEventHandler[str]):

    def on_message_delta(self, delta: "MessageDeltaChunk") -> Optional[str]:
        return f"Text delta received: {delta.text}"

    def on_thread_message(self, message: "ThreadMessage") -> Optional[str]:
        return f"ThreadMessage created. ID: {message.id}, Status: {message.status}"

    def on_thread_run(self, run: "ThreadRun") -> Optional[str]:
        return f"ThreadRun status: {run.status}"

    def on_run_step(self, step: "RunStep") -> Optional[str]:
        return f"RunStep type: {step.type}, Status: {step.status}"

    def on_error(self, data: str) -> Optional[str]:
        return f"An error occurred. Data: {data}"

    def on_done(self) -> Optional[str]:
        return "Stream completed."

    def on_unhandled_event(self, event_type: str, event_data: Any) -> Optional[str]:
        return f"Unhandled Event Type: {event_type}, Data: {event_data}"


# [END stream_event_handler]


with agents_client:
    # Create an agent and run stream with event handler
    agent = agents_client.create_agent(
        model=os.environ["MODEL_DEPLOYMENT_NAME"], name="my-agent", instructions="You are a helpful agent"
    )
    print(f"Created agent, agent ID {agent.id}")

    thread = agents_client.threads.create()
    print(f"Created thread, thread ID {thread.id}")

    message = agents_client.messages.create(thread_id=thread.id, role="user", content="Hello, tell me a joke")
    print(f"Created message, message ID {message.id}")

    # [START create_stream]
    with agents_client.runs.stream(thread_id=thread.id, agent_id=agent.id, event_handler=MyEventHandler()) as stream:
        for event_type, event_data, func_return in stream:
            print(f"Received data.")
            print(f"Streaming receive Event Type: {event_type}")
            print(f"Event Data: {str(event_data)[:100]}...")
            print(f"Event Function return: {func_return}\n")
    # [END create_stream]

    agents_client.delete_agent(agent.id)
    print("Deleted agent")

    messages = agents_client.messages.list(thread_id=thread.id)
    for msg in messages:
        if msg.text_messages:
            last_text = msg.text_messages[-1]
            print(f"{msg.role}: {last_text.text.value}")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.batch import BatchManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-batch
# USAGE
    python pool_create_virtual_machine_configuration_managed_os_disk.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = BatchManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="subid",
    )

    response = client.pool.create(
        resource_group_name="default-azurebatch-japaneast",
        account_name="sampleacct",
        pool_name="testpool",
        parameters={
            "properties": {
                "deploymentConfiguration": {
                    "virtualMachineConfiguration": {
                        "imageReference": {
                            "offer": "windowsserver",
                            "publisher": "microsoftwindowsserver",
                            "sku": "2022-datacenter-smalldisk",
                        },
                        "nodeAgentSkuId": "batch.node.windows amd64",
                        "osDisk": {
                            "caching": "ReadWrite",
                            "diskSizeGB": 100,
                            "managedDisk": {"storageAccountType": "StandardSSD_LRS"},
                            "writeAcceleratorEnabled": False,
                        },
                    }
                },
                "scaleSettings": {"fixedScale": {"targetDedicatedNodes": 1, "targetLowPriorityNodes": 0}},
                "vmSize": "Standard_d2s_v3",
            }
        },
    )
    print(response)


# x-ms-original-file: specification/batch/resource-manager/Microsoft.Batch/stable/2024-07-01/examples/PoolCreate_VirtualMachineConfiguration_ManagedOSDisk.json
if __name__ == "__main__":
    main()



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------

import hashlib
import urllib
import base64
import hmac
from urllib.parse import ParseResult, urlparse
from typing import Union
from azure.core.credentials import AzureKeyCredential
from azure.core.pipeline.policies import SansIOHTTPPolicy
from .utils import get_current_utc_time


class HMACCredentialsPolicy(SansIOHTTPPolicy):
    """Implementation of HMAC authentication policy.

    :param str host: The host of the endpoint url for Azure Communication Service resource
    :param access_key: The access key we use to authenticate to the service
    :type access_key: str or AzureKeyCredential
    :param bool decode_url: `True` if there is a need to decode the url. Default value is `False`
    """

    def __init__(
        self,
        host: str,
        access_key: Union[str, AzureKeyCredential],
        decode_url: bool = False,
    ) -> None:
        super(HMACCredentialsPolicy, self).__init__()

        if host.startswith("https://"):
            self._host = host.replace("https://", "")

        if host.startswith("http://"):
            self._host = host.replace("http://", "")

        self._access_key = access_key
        self._decode_url = decode_url

    def _compute_hmac(
        self, value  # type: str
    ):
        if isinstance(self._access_key, AzureKeyCredential):
            decoded_secret = base64.b64decode(self._access_key.key)
        else:
            decoded_secret = base64.b64decode(self._access_key)

        digest = hmac.new(decoded_secret, value.encode("utf-8"), hashlib.sha256).digest()

        return base64.b64encode(digest).decode("utf-8")

    def _sign_request(self, request):
        verb = request.http_request.method.upper()

        # Get the path and query from url, which looks like https://host/path/query
        parsed_url: ParseResult = urlparse(request.http_request.url)
        query_url = parsed_url.path

        if parsed_url.query:
            query_url += "?" + parsed_url.query

        # Need URL() to get a correct encoded key value, from "%3A" to ":", when transport is in type AioHttpTransport.
        # There's a similar scenario in azure-storage-blob and azure-appconfiguration, the check logic is from there.
        try:
            from yarl import URL
            from azure.core.pipeline.transport import (  # pylint:disable=non-abstract-transport-import
                AioHttpTransport,
            )

            if (
                isinstance(request.context.transport, AioHttpTransport)
                or isinstance(
                    getattr(request.context.transport, "_transport", None),
                    AioHttpTransport,
                )
                or isinstance(
                    getattr(
                        getattr(request.context.transport, "_transport", None),
                        "_transport",
                        None,
                    ),
                    AioHttpTransport,
                )
            ):
                query_url = str(URL(query_url))
        except (ImportError, TypeError):
            pass

        if self._decode_url:
            query_url = urllib.parse.unquote(query_url)

        signed_headers = "x-ms-date;host;x-ms-content-sha256"

        utc_now = get_current_utc_time()
        if request.http_request.body is None:
            request.http_request.body = ""
        content_digest = hashlib.sha256((request.http_request.body.encode("utf-8"))).digest()
        content_hash = base64.b64encode(content_digest).decode("utf-8")

        string_to_sign = verb + "\n" + query_url + "\n" + utc_now + ";" + self._host + ";" + content_hash

        signature = self._compute_hmac(string_to_sign)

        signature_header = {
            "x-ms-date": utc_now,
            "x-ms-content-sha256": content_hash,
            "x-ms-return-client-request-id": "true",
            "Authorization": "HMAC-SHA256 SignedHeaders=" + signed_headers + "&Signature=" + signature,
        }

        request.http_request.headers.update(signature_header)

        return request

    def on_request(self, request):
        self._sign_request(request)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models_py3 import (  # type: ignore
    AbnormalTimePeriod,
    AnalysisData,
    AnalysisDefinition,
    ApiDefinitionInfo,
    AppServiceEnvironment,
    AppServicePlan,
    AppServicePlanCollection,
    ApplicationStack,
    ApplicationStackCollection,
    AutoHealActions,
    AutoHealCustomAction,
    AutoHealRules,
    AutoHealTriggers,
    BillingMeter,
    BillingMeterCollection,
    Capability,
    Certificate,
    CertificateCollection,
    CertificatePatchResource,
    CloningInfo,
    ConnStringInfo,
    CorsSettings,
    CsmMoveResourceEnvelope,
    CsmOperationCollection,
    CsmOperationDescription,
    CsmOperationDescriptionProperties,
    CsmOperationDisplay,
    CsmUsageQuota,
    CsmUsageQuotaCollection,
    DataSource,
    DataTableResponseColumn,
    DataTableResponseObject,
    DefaultErrorResponse,
    DefaultErrorResponseError,
    DefaultErrorResponseErrorDetailsItem,
    DeletedSite,
    DeletedWebAppCollection,
    DeploymentLocations,
    DetectorAbnormalTimePeriod,
    DetectorDefinition,
    DetectorInfo,
    DetectorResponse,
    DetectorResponseCollection,
    DiagnosticAnalysis,
    DiagnosticAnalysisCollection,
    DiagnosticCategory,
    DiagnosticCategoryCollection,
    DiagnosticData,
    DiagnosticDetectorCollection,
    DiagnosticDetectorResponse,
    DiagnosticMetricSample,
    DiagnosticMetricSet,
    Dimension,
    ErrorEntity,
    Experiments,
    GeoRegion,
    GeoRegionCollection,
    GlobalCsmSkuDescription,
    HandlerMapping,
    HostNameSslState,
    HostingEnvironmentDeploymentInfo,
    HostingEnvironmentProfile,
    HybridConnection,
    HybridConnectionKey,
    Identifier,
    IdentifierCollection,
    IpSecurityRestriction,
    LocalizableString,
    ManagedServiceIdentity,
    MetricAvailability,
    MetricSpecification,
    NameIdentifier,
    NameValuePair,
    NetworkAccessControlEntry,
    Operation,
    PremierAddOnOffer,
    PremierAddOnOfferCollection,
    ProxyOnlyResource,
    PushSettings,
    RampUpRule,
    Recommendation,
    RecommendationCollection,
    RecommendationRule,
    Rendering,
    RequestsBasedTrigger,
    Resource,
    ResourceHealthMetadata,
    ResourceHealthMetadataCollection,
    ResourceMetric,
    ResourceMetricAvailability,
    ResourceMetricCollection,
    ResourceMetricDefinition,
    ResourceMetricDefinitionCollection,
    ResourceMetricName,
    ResourceMetricProperty,
    ResourceMetricValue,
    ResourceNameAvailability,
    ResourceNameAvailabilityRequest,
    ResponseMetaData,
    ServiceSpecification,
    Site,
    SiteConfig,
    SiteLimits,
    SiteMachineKey,
    SkuCapacity,
    SkuDescription,
    SkuInfos,
    SlotSwapStatus,
    SlowRequestsBasedTrigger,
    SnapshotRecoveryRequest,
    SnapshotRecoveryTarget,
    Solution,
    SourceControl,
    SourceControlCollection,
    StackMajorVersion,
    StackMinorVersion,
    StampCapacity,
    StatusCodesBasedTrigger,
    User,
    ValidateRequest,
    ValidateResponse,
    ValidateResponseError,
    VirtualApplication,
    VirtualDirectory,
    VirtualIPMapping,
    VirtualNetworkProfile,
    VnetGateway,
    VnetInfo,
    VnetParameters,
    VnetRoute,
    VnetValidationFailureDetails,
    VnetValidationTestFailure,
    WebAppCollection,
    WorkerPool,
)

from ._web_site_management_client_enums import (  # type: ignore
    AccessControlEntryAction,
    AppServicePlanRestrictions,
    AutoHealActionType,
    Channels,
    CheckNameResourceTypes,
    ComputeModeOptions,
    ConnectionStringType,
    Enum0,
    Enum1,
    HostType,
    HostingEnvironmentStatus,
    InAvailabilityReasonType,
    InternalLoadBalancingMode,
    IssueType,
    KeyVaultSecretStatus,
    ManagedPipelineMode,
    ManagedServiceIdentityType,
    NotificationLevel,
    OperationStatus,
    ProvisioningState,
    RenderingType,
    ResourceScopeType,
    RouteType,
    ScmType,
    SiteAvailabilityState,
    SiteLoadBalancing,
    SkuName,
    SolutionType,
    SslState,
    StatusOptions,
    SupportedTlsVersions,
    UsageState,
    ValidateResourceTypes,
    WorkerSizeOptions,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AbnormalTimePeriod",
    "AnalysisData",
    "AnalysisDefinition",
    "ApiDefinitionInfo",
    "AppServiceEnvironment",
    "AppServicePlan",
    "AppServicePlanCollection",
    "ApplicationStack",
    "ApplicationStackCollection",
    "AutoHealActions",
    "AutoHealCustomAction",
    "AutoHealRules",
    "AutoHealTriggers",
    "BillingMeter",
    "BillingMeterCollection",
    "Capability",
    "Certificate",
    "CertificateCollection",
    "CertificatePatchResource",
    "CloningInfo",
    "ConnStringInfo",
    "CorsSettings",
    "CsmMoveResourceEnvelope",
    "CsmOperationCollection",
    "CsmOperationDescription",
    "CsmOperationDescriptionProperties",
    "CsmOperationDisplay",
    "CsmUsageQuota",
    "CsmUsageQuotaCollection",
    "DataSource",
    "DataTableResponseColumn",
    "DataTableResponseObject",
    "DefaultErrorResponse",
    "DefaultErrorResponseError",
    "DefaultErrorResponseErrorDetailsItem",
    "DeletedSite",
    "DeletedWebAppCollection",
    "DeploymentLocations",
    "DetectorAbnormalTimePeriod",
    "DetectorDefinition",
    "DetectorInfo",
    "DetectorResponse",
    "DetectorResponseCollection",
    "DiagnosticAnalysis",
    "DiagnosticAnalysisCollection",
    "DiagnosticCategory",
    "DiagnosticCategoryCollection",
    "DiagnosticData",
    "DiagnosticDetectorCollection",
    "DiagnosticDetectorResponse",
    "DiagnosticMetricSample",
    "DiagnosticMetricSet",
    "Dimension",
    "ErrorEntity",
    "Experiments",
    "GeoRegion",
    "GeoRegionCollection",
    "GlobalCsmSkuDescription",
    "HandlerMapping",
    "HostNameSslState",
    "HostingEnvironmentDeploymentInfo",
    "HostingEnvironmentProfile",
    "HybridConnection",
    "HybridConnectionKey",
    "Identifier",
    "IdentifierCollection",
    "IpSecurityRestriction",
    "LocalizableString",
    "ManagedServiceIdentity",
    "MetricAvailability",
    "MetricSpecification",
    "NameIdentifier",
    "NameValuePair",
    "NetworkAccessControlEntry",
    "Operation",
    "PremierAddOnOffer",
    "PremierAddOnOfferCollection",
    "ProxyOnlyResource",
    "PushSettings",
    "RampUpRule",
    "Recommendation",
    "RecommendationCollection",
    "RecommendationRule",
    "Rendering",
    "RequestsBasedTrigger",
    "Resource",
    "ResourceHealthMetadata",
    "ResourceHealthMetadataCollection",
    "ResourceMetric",
    "ResourceMetricAvailability",
    "ResourceMetricCollection",
    "ResourceMetricDefinition",
    "ResourceMetricDefinitionCollection",
    "ResourceMetricName",
    "ResourceMetricProperty",
    "ResourceMetricValue",
    "ResourceNameAvailability",
    "ResourceNameAvailabilityRequest",
    "ResponseMetaData",
    "ServiceSpecification",
    "Site",
    "SiteConfig",
    "SiteLimits",
    "SiteMachineKey",
    "SkuCapacity",
    "SkuDescription",
    "SkuInfos",
    "SlotSwapStatus",
    "SlowRequestsBasedTrigger",
    "SnapshotRecoveryRequest",
    "SnapshotRecoveryTarget",
    "Solution",
    "SourceControl",
    "SourceControlCollection",
    "StackMajorVersion",
    "StackMinorVersion",
    "StampCapacity",
    "StatusCodesBasedTrigger",
    "User",
    "ValidateRequest",
    "ValidateResponse",
    "ValidateResponseError",
    "VirtualApplication",
    "VirtualDirectory",
    "VirtualIPMapping",
    "VirtualNetworkProfile",
    "VnetGateway",
    "VnetInfo",
    "VnetParameters",
    "VnetRoute",
    "VnetValidationFailureDetails",
    "VnetValidationTestFailure",
    "WebAppCollection",
    "WorkerPool",
    "AccessControlEntryAction",
    "AppServicePlanRestrictions",
    "AutoHealActionType",
    "Channels",
    "CheckNameResourceTypes",
    "ComputeModeOptions",
    "ConnectionStringType",
    "Enum0",
    "Enum1",
    "HostType",
    "HostingEnvironmentStatus",
    "InAvailabilityReasonType",
    "InternalLoadBalancingMode",
    "IssueType",
    "KeyVaultSecretStatus",
    "ManagedPipelineMode",
    "ManagedServiceIdentityType",
    "NotificationLevel",
    "OperationStatus",
    "ProvisioningState",
    "RenderingType",
    "ResourceScopeType",
    "RouteType",
    "ScmType",
    "SiteAvailabilityState",
    "SiteLoadBalancing",
    "SkuName",
    "SolutionType",
    "SslState",
    "StatusOptions",
    "SupportedTlsVersions",
    "UsageState",
    "ValidateResourceTypes",
    "WorkerSizeOptions",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import pytest
import unittest
from unittest.mock import patch
from azure.appconfiguration.provider._client_manager import ConfigurationClientManager


class MockClient:

    def __init__(self, endpoint, connection_string, credential, retry_total, retry_backoff):
        self.endpoint = endpoint
        self.connection_string = connection_string
        self.credential = credential
        self.retry_total = retry_total
        self.retry_backoff = retry_backoff


@pytest.mark.usefixtures("caplog")
class TestConfigurationClientManagerLoadBalance(unittest.TestCase):

    @patch("azure.appconfiguration.provider._client_manager.find_auto_failover_endpoints")
    @patch("azure.appconfiguration.provider._client_manager.AzureAppConfigurationClient.from_connection_string")
    def test_find_active_clients(self, mock_client, mock_find_auto_failover_endpoints):
        # Single endpoint test no load balancing
        endpoint = "https://fake.endpoint"
        connection_string = "Endpoint=https://fake.endpoint/;Id=fake_id;Secret=fake_secret"

        mock_find_auto_failover_endpoints.return_value = []

        with ConfigurationClientManager(connection_string, endpoint, None, "", 0, 0, True, 0, 0, False) as manager:
            assert manager.get_next_active_client() is None

            manager.find_active_clients()
            assert len(manager._active_clients) == 1

        # Multiple endpoint test no load balancing
        failover_endpoints = ["https://fake.endpoint2", "https://fake.endpoint3"]
        mock_find_auto_failover_endpoints.return_value = failover_endpoints

        manager = ConfigurationClientManager(connection_string, endpoint, None, "", 0, 0, True, 0, 0, False)
        manager.refresh_clients()

        manager.find_active_clients()

        assert len(manager._active_clients) == 3
        assert manager._active_clients[0].endpoint in failover_endpoints + [endpoint]
        assert manager._active_clients[1].endpoint in failover_endpoints + [endpoint]
        assert manager._active_clients[2].endpoint in failover_endpoints + [endpoint]

        # Single endpoint test load balancing
        mock_find_auto_failover_endpoints.return_value = []

        manager = ConfigurationClientManager(connection_string, endpoint, None, "", 0, 0, True, 0, 0, True)
        manager.refresh_clients()

        manager.find_active_clients()
        assert len(manager._active_clients) == 1

        # Multiple endpoint test load balancing
        mock_find_auto_failover_endpoints.return_value = ["https://fake.endpoint2", "https://fake.endpoint3"]

        manager = ConfigurationClientManager(connection_string, endpoint, None, "", 0, 0, True, 0, 0, True)
        manager.refresh_clients()

        manager.find_active_clients()
        assert len(manager._active_clients) == 3
        endpoint_list = [client.endpoint for client in manager._active_clients]
        next_client = manager.get_next_active_client()
        assert next_client is not None
        assert next_client.endpoint == endpoint_list[0]
        manager.find_active_clients()
        assert manager._active_clients[0].endpoint == endpoint_list[1]
        assert manager._active_clients[1].endpoint == endpoint_list[2]
        assert manager._active_clients[2].endpoint == endpoint_list[0]
        next_client = manager.get_next_active_client()
        assert next_client is not None
        assert next_client.endpoint == endpoint_list[1]
        manager.find_active_clients()
        manager.get_next_active_client()
        manager.find_active_clients()
        assert manager._active_clients[0].endpoint == endpoint_list[0]
        assert manager._active_clients[1].endpoint == endpoint_list[1]
        assert manager._active_clients[2].endpoint == endpoint_list[2]
        next_client = manager.get_next_active_client()
        assert next_client is not None
        assert next_client.endpoint == endpoint_list[0]

        manager.close()

    @patch("azure.appconfiguration.provider._client_manager.find_auto_failover_endpoints")
    @patch("azure.appconfiguration.provider._client_manager.AzureAppConfigurationClient")
    def test_find_active_clients_entra_id(self, mock_client, mock_find_auto_failover_endpoints):
        # Single endpoint test no load balancing
        endpoint = "https://fake.endpoint"

        mock_find_auto_failover_endpoints.return_value = []

        with ConfigurationClientManager(None, endpoint, "fake-credential", "", 0, 0, True, 0, 0, False) as manager:
            assert manager.get_next_active_client() is None

            manager.find_active_clients()
            assert len(manager._active_clients) == 1

        # Multiple endpoint test no load balancing
        failover_endpoints = ["https://fake.endpoint2", "https://fake.endpoint3"]
        mock_find_auto_failover_endpoints.return_value = failover_endpoints

        manager = ConfigurationClientManager(None, endpoint, "fake-credential", "", 0, 0, True, 0, 0, True)
        manager.refresh_clients()

        manager.find_active_clients()

        assert len(manager._active_clients) == 3
        assert manager._active_clients[0].endpoint in failover_endpoints + [endpoint]
        assert manager._active_clients[1].endpoint in failover_endpoints + [endpoint]
        assert manager._active_clients[2].endpoint in failover_endpoints + [endpoint]

        # Single endpoint test load balancing
        mock_find_auto_failover_endpoints.return_value = []

        manager = ConfigurationClientManager(None, endpoint, "fake-credential", "", 0, 0, True, 0, 0, True)
        manager.refresh_clients()

        manager.find_active_clients()
        assert len(manager._active_clients) == 1

        # Multiple endpoint test load balancing
        mock_find_auto_failover_endpoints.return_value = ["https://fake.endpoint2", "https://fake.endpoint3"]

        manager = ConfigurationClientManager(None, endpoint, "fake-credential", "", 0, 0, True, 0, 0, True)
        manager.refresh_clients()

        manager.find_active_clients()
        assert len(manager._active_clients) == 3
        endpoint_list = [client.endpoint for client in manager._active_clients]
        next_client = manager.get_next_active_client()
        assert next_client is not None
        assert next_client.endpoint == endpoint_list[0]
        manager.find_active_clients()
        assert manager._active_clients[0].endpoint == endpoint_list[1]
        assert manager._active_clients[1].endpoint == endpoint_list[2]
        assert manager._active_clients[2].endpoint == endpoint_list[0]
        next_client = manager.get_next_active_client()
        assert next_client is not None
        assert next_client.endpoint == endpoint_list[1]
        manager.find_active_clients()
        manager.get_next_active_client()
        manager.find_active_clients()
        assert manager._active_clients[0].endpoint == endpoint_list[0]
        assert manager._active_clients[1].endpoint == endpoint_list[1]
        assert manager._active_clients[2].endpoint == endpoint_list[2]
        next_client = manager.get_next_active_client()
        assert next_client is not None
        assert next_client.endpoint == endpoint_list[0]

        manager.close()



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: send_email_to_single_recipient_sample_async.py
DESCRIPTION:
    This sample demonstrates sending an email to a single recipient. The Email client is 
    authenticated using a connection string.
USAGE:
    python send_email_to_single_recipient_sample_async.py
    Set the environment variable with your own value before running the sample:
    1) COMMUNICATION_CONNECTION_STRING_EMAIL - the connection string in your ACS resource
    2) SENDER_ADDRESS - the address found in the linked domain that will send the email
    3) RECIPIENT_ADDRESS - the address that will receive the email
"""

import os
import sys
import asyncio
from azure.core.exceptions import HttpResponseError
from azure.communication.email.aio import EmailClient

sys.path.append("..")


class EmailSingleRecipientSampleAsync(object):

    connection_string = os.getenv("COMMUNICATION_CONNECTION_STRING_EMAIL")
    sender_address = os.getenv("SENDER_ADDRESS")
    recipient_address = os.getenv("RECIPIENT_ADDRESS")

    async def send_email_to_single_recipient_async(self):
        # creating the email client
        email_client = EmailClient.from_connection_string(self.connection_string)

        # creating the email message
        message = {
            "content": {
                "subject": "This is the subject",
                "plainText": "This is the body",
                "html": "html><h1>This is the body</h1></html>",
            },
            "recipients": {"to": [{"address": self.recipient_address, "displayName": "Customer Name"}]},
            "senderAddress": self.sender_address,
        }

        async with email_client:
            try:
                # sending the email message
                poller = await email_client.begin_send(message)
                response = await poller.result()
                print("Operation ID: " + response["id"])
            except HttpResponseError as ex:
                print(ex)
                pass


if __name__ == "__main__":
    sample = EmailSingleRecipientSampleAsync()
    asyncio.run(sample.send_email_to_single_recipient_async())



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------
"""
This module is the requests implementation of Pipeline ABC
"""
from __future__ import annotations
from typing import Dict, Any, Optional, cast, TYPE_CHECKING
import logging
import time

from ...transport import AsyncHttpTransport
from ...rest import AsyncHttpResponse, HttpRequest
from ...exceptions import (
    BaseError,
    ClientAuthenticationError,
    ServiceRequestError,
)
from ._base_async import AsyncHTTPPolicy
from ._retry import RetryPolicyBase

if TYPE_CHECKING:
    from ...runtime.pipeline import PipelineRequest, PipelineResponse

_LOGGER = logging.getLogger(__name__)


class AsyncRetryPolicy(RetryPolicyBase, AsyncHTTPPolicy[HttpRequest, AsyncHttpResponse]):
    """Async flavor of the retry policy.

    The async retry policy in the pipeline can be configured directly, or tweaked on a per-call basis.

    :keyword int retry_total: Total number of retries to allow. Takes precedence over other counts.
     Default value is 10.

    :keyword int retry_connect: How many connection-related errors to retry on.
     These are errors raised before the request is sent to the remote server,
     which we assume has not triggered the server to process the request. Default value is 3.

    :keyword int retry_read: How many times to retry on read errors.
     These errors are raised after the request was sent to the server, so the
     request may have side-effects. Default value is 3.

    :keyword int retry_status: How many times to retry on bad status codes. Default value is 3.

    :keyword float retry_backoff_factor: A backoff factor to apply between attempts after the second try
     (most errors are resolved immediately by a second try without a delay).
     Retry policy will sleep for: `{backoff factor} * (2 ** ({number of total retries} - 1))`
     seconds. If the backoff_factor is 0.1, then the retry will sleep
     for [0.0s, 0.2s, 0.4s, ...] between retries. The default value is 0.8.

    :keyword int retry_backoff_max: The maximum back off time. Default value is 120 seconds (2 minutes).
    """

    async def _sleep_for_retry(
        self,
        response: PipelineResponse[HttpRequest, AsyncHttpResponse],
        transport: AsyncHttpTransport[HttpRequest, AsyncHttpResponse],
    ) -> bool:
        """Sleep based on the Retry-After response header value.

        :param response: The PipelineResponse object.
        :type response: ~corehttp.runtime.pipeline.PipelineResponse
        :param transport: The HTTP transport type.
        :type transport: ~corehttp.transport.AsyncHttpTransport
        :return: Whether the retry-after value was found.
        :rtype: bool
        """
        retry_after = self.get_retry_after(response)
        if retry_after:
            await transport.sleep(retry_after)
            return True
        return False

    async def _sleep_backoff(
        self, settings: Dict[str, Any], transport: AsyncHttpTransport[HttpRequest, AsyncHttpResponse]
    ) -> None:
        """Sleep using exponential backoff. Immediately returns if backoff is 0.

        :param dict settings: The retry settings.
        :param transport: The HTTP transport type.
        :type transport: ~corehttp.transport.AsyncHttpTransport
        """
        backoff = self.get_backoff_time(settings)
        if backoff <= 0:
            return
        await transport.sleep(backoff)

    async def sleep(
        self,
        settings: Dict[str, Any],
        transport: AsyncHttpTransport[HttpRequest, AsyncHttpResponse],
        response: Optional[PipelineResponse[HttpRequest, AsyncHttpResponse]] = None,
    ) -> None:
        """Sleep between retry attempts.

        This method will respect a server's ``Retry-After`` response header
        and sleep the duration of the time requested. If that is not present, it
        will use an exponential backoff. By default, the backoff factor is 0 and
        this method will return immediately.

        :param dict settings: The retry settings.
        :param transport: The HTTP transport type.
        :type transport: ~corehttp.transport.AsyncHttpTransport
        :param response: The PipelineResponse object.
        :type response: ~corehttp.runtime.pipeline.PipelineResponse
        """
        if response:
            slept = await self._sleep_for_retry(response, transport)
            if slept:
                return
        await self._sleep_backoff(settings, transport)

    async def send(self, request: PipelineRequest[HttpRequest]) -> PipelineResponse[HttpRequest, AsyncHttpResponse]:
        """Uses the configured retry policy to send the request to the next policy in the pipeline.

        :param request: The PipelineRequest object
        :type request: ~corehttp.runtime.pipeline.PipelineRequest
        :return: Returns the PipelineResponse or raises error if maximum retries exceeded.
        :rtype: ~corehttp.runtime.pipeline.PipelineResponse
        :raise: ~corehttp.exceptions.BaseError if maximum retries exceeded.
        :raise: ~corehttp.exceptions.ClientAuthenticationError if authentication fails
        """
        retry_active = True
        response = None
        retry_settings = self.configure_retries(request.context.options)
        self._configure_positions(request, retry_settings)

        absolute_timeout = retry_settings["timeout"]
        is_response_error = True

        while retry_active:
            start_time = time.time()
            # PipelineContext types transport as a Union of HttpTransport and AsyncHttpTransport, but
            # here we know that this is an AsyncHttpTransport.
            # The correct fix is to make PipelineContext generic, but that's a breaking change and a lot of
            # generic to update in Pipeline, PipelineClient, PipelineRequest, PipelineResponse, etc.
            transport: AsyncHttpTransport[HttpRequest, AsyncHttpResponse] = cast(
                AsyncHttpTransport[HttpRequest, AsyncHttpResponse], request.context.transport
            )
            try:
                self._configure_timeout(request, absolute_timeout, is_response_error)
                request.context["retry_count"] = len(retry_settings["history"])
                response = await self.next.send(request)
                if self.is_retry(retry_settings, response):
                    retry_active = self.increment(retry_settings, response=response)
                    if retry_active:
                        await self.sleep(
                            retry_settings,
                            transport,
                            response=response,
                        )
                        is_response_error = True
                        continue
                break
            except ClientAuthenticationError:
                # the authentication policy failed such that the client's request can't
                # succeed--we'll never have a response to it, so propagate the exception
                raise
            except BaseError as err:
                if absolute_timeout > 0 and self._is_method_retryable(retry_settings, request.http_request):
                    retry_active = self.increment(retry_settings, response=request, error=err)
                    if retry_active:
                        await self.sleep(retry_settings, transport)
                        if isinstance(err, ServiceRequestError):
                            is_response_error = False
                        else:
                            is_response_error = True
                        continue
                raise err
            finally:
                end_time = time.time()
                if absolute_timeout:
                    absolute_timeout -= end_time - start_time
        if not response:
            raise BaseError("Maximum retries exceeded.")

        self.update_context(response.context, retry_settings)
        return response



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
from __future__ import annotations
import uuid
from base64 import b64decode
from datetime import datetime
from typing import cast, Union, Any, Optional, Dict, TypeVar, Generic
from .utils._utils import _convert_to_isoformat, TZ_UTC
from .utils._messaging_shared import _get_json_content
from .serialization import NULL


__all__ = ["CloudEvent"]


_Unset: Any = object()

DataType = TypeVar("DataType")


class CloudEvent(Generic[DataType]):
    """Properties of the CloudEvent 1.0 Schema.
    All required parameters must be populated in order to send to Azure.

    :param source: Required. Identifies the context in which an event happened. The combination of id and source must
     be unique for each distinct event. If publishing to a domain topic, source must be the domain topic name.
    :type source: str
    :param type: Required. Type of event related to the originating occurrence.
    :type type: str
    :keyword specversion: Optional. The version of the CloudEvent spec. Defaults to "1.0"
    :paramtype specversion: str
    :keyword data: Optional. Event data specific to the event type.
    :paramtype data: object
    :keyword time: Optional. The time (in UTC) the event was generated.
    :paramtype time: ~datetime.datetime
    :keyword dataschema: Optional. Identifies the schema that data adheres to.
    :paramtype dataschema: str
    :keyword datacontenttype: Optional. Content type of data value.
    :paramtype datacontenttype: str
    :keyword subject: Optional. This describes the subject of the event in the context of the event producer
     (identified by source).
    :paramtype subject: str
    :keyword id: Optional. An identifier for the event. The combination of id and source must be
     unique for each distinct event. If not provided, a random UUID will be generated and used.
    :paramtype id: Optional[str]
    :keyword extensions: Optional. A CloudEvent MAY include any number of additional context attributes
     with distinct names represented as key - value pairs. Each extension must be alphanumeric, lower cased
     and must not exceed the length of 20 characters.
    :paramtype extensions: Optional[dict]
    """

    source: str
    """Identifies the context in which an event happened. The combination of id and source must
       be unique for each distinct event. If publishing to a domain topic, source must be the domain topic name."""

    type: str
    """Type of event related to the originating occurrence."""

    specversion: str = "1.0"
    """The version of the CloudEvent spec. Defaults to "1.0" """

    id: str
    """An identifier for the event. The combination of id and source must be
       unique for each distinct event. If not provided, a random UUID will be generated and used."""

    data: Optional[DataType]
    """Event data specific to the event type."""

    time: Optional[datetime]
    """The time (in UTC) the event was generated."""

    dataschema: Optional[str]
    """Identifies the schema that data adheres to."""

    datacontenttype: Optional[str]
    """Content type of data value."""

    subject: Optional[str]
    """This describes the subject of the event in the context of the event producer
       (identified by source)"""

    extensions: Optional[Dict[str, Any]]
    """A CloudEvent MAY include any number of additional context attributes
       with distinct names represented as key - value pairs. Each extension must be alphanumeric, lower cased
       and must not exceed the length of 20 characters."""

    def __init__(
        self,
        source: str,
        type: str,  # pylint: disable=redefined-builtin
        *,
        specversion: Optional[str] = None,
        id: Optional[str] = None,  # pylint: disable=redefined-builtin
        time: Optional[datetime] = _Unset,
        datacontenttype: Optional[str] = None,
        dataschema: Optional[str] = None,
        subject: Optional[str] = None,
        data: Optional[DataType] = None,
        extensions: Optional[Dict[str, Any]] = None,
        **kwargs: Any,
    ) -> None:
        self.source: str = source
        self.type: str = type

        if specversion:
            self.specversion: str = specversion
        self.id: str = id if id else str(uuid.uuid4())

        self.time: Optional[datetime]
        if time is _Unset:
            self.time = datetime.now(TZ_UTC)
        else:
            self.time = time

        self.datacontenttype: Optional[str] = datacontenttype
        self.dataschema: Optional[str] = dataschema
        self.subject: Optional[str] = subject
        self.data: Optional[DataType] = data

        self.extensions: Optional[Dict[str, Any]] = extensions
        if self.extensions:
            for key in self.extensions.keys():
                if not key.islower() or not key.isalnum():
                    raise ValueError("Extension attributes should be lower cased and alphanumeric.")

        if kwargs:
            remaining = ", ".join(kwargs.keys())
            raise ValueError(
                f"Unexpected keyword arguments {remaining}. "
                + "Any extension attributes must be passed explicitly using extensions."
            )

    def __repr__(self) -> str:
        return "CloudEvent(source={}, type={}, specversion={}, id={}, time={})".format(
            self.source, self.type, self.specversion, self.id, self.time
        )[:1024]

    @classmethod
    def from_dict(cls, event: Dict[str, Any]) -> CloudEvent[DataType]:
        """Returns the deserialized CloudEvent object when a dict is provided.

        :param event: The dict representation of the event which needs to be deserialized.
        :type event: dict
        :rtype: CloudEvent
        :return: The deserialized CloudEvent object.
        """
        kwargs: Dict[str, Any] = {}
        reserved_attr = [
            "data",
            "data_base64",
            "id",
            "source",
            "type",
            "specversion",
            "time",
            "dataschema",
            "datacontenttype",
            "subject",
        ]

        if "data" in event and "data_base64" in event:
            raise ValueError("Invalid input. Only one of data and data_base64 must be present.")

        if "data" in event:
            data = event.get("data")
            kwargs["data"] = data if data is not None else NULL
        elif "data_base64" in event:
            kwargs["data"] = b64decode(cast(Union[str, bytes], event.get("data_base64")))

        for item in ["datacontenttype", "dataschema", "subject"]:
            if item in event:
                val = event.get(item)
                kwargs[item] = val if val is not None else NULL

        extensions = {k: v for k, v in event.items() if k not in reserved_attr}
        if extensions:
            kwargs["extensions"] = extensions

        try:
            event_obj = cls(
                id=event.get("id"),
                source=event["source"],
                type=event["type"],
                specversion=event.get("specversion"),
                time=_convert_to_isoformat(event.get("time")),
                **kwargs,
            )
        except KeyError as err:
            # https://github.com/cloudevents/spec Cloud event spec requires source, type,
            # specversion. We autopopulate everything other than source, type.
            # So we will assume the KeyError is coming from source/type access.
            if all(
                key in event
                for key in (
                    "subject",
                    "eventType",
                    "data",
                    "dataVersion",
                    "id",
                    "eventTime",
                )
            ):
                raise ValueError(
                    "The event you are trying to parse follows the Eventgrid Schema. You can parse"
                    + " EventGrid events using EventGridEvent.from_dict method in the azure-eventgrid library."
                ) from err
            raise ValueError(
                "The event does not conform to the cloud event spec https://github.com/cloudevents/spec."
                + " The `source` and `type` params are required."
            ) from err
        return event_obj

    @classmethod
    def from_json(cls, event: Any) -> CloudEvent[DataType]:
        """Returns the deserialized CloudEvent object when a json payload is provided.

        :param event: The json string that should be converted into a CloudEvent. This can also be
         a storage QueueMessage, eventhub's EventData or ServiceBusMessage
        :type event: object
        :rtype: CloudEvent
        :return: The deserialized CloudEvent object.
        :raises ValueError: If the provided JSON is invalid.
        """
        dict_event = _get_json_content(event)
        return CloudEvent.from_dict(dict_event)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import pytest

from azure.servicebus import ServiceBusClient, ServiceBusMessage
from opentelemetry.trace import SpanKind


class TestServiceBusTracing:
    def _verify_span_attributes(self, *, span):
        # Ensure all attributes are set and have a value.
        for attr in span.attributes:
            assert span.attributes[attr] is not None and span.attributes[attr] != ""

    def _verify_message(self, *, span, dest, server_address):
        assert span.name == "ServiceBus.message"
        assert span.kind == SpanKind.PRODUCER
        self._verify_span_attributes(span=span)
        assert span.attributes["az.namespace"] == "Microsoft.ServiceBus"
        assert span.attributes["messaging.system"] == "servicebus"
        assert span.attributes["messaging.destination.name"] == dest
        assert span.attributes["server.address"] == server_address

    def _verify_send(self, *, span, dest, server_address, message_count):
        assert span.name == "ServiceBus.send"
        assert span.kind == SpanKind.CLIENT
        self._verify_span_attributes(span=span)
        assert span.attributes["az.namespace"] == "Microsoft.ServiceBus"
        assert span.attributes["messaging.system"] == "servicebus"
        assert span.attributes["messaging.destination.name"] == dest
        assert span.attributes["messaging.operation"] == "publish"
        assert span.attributes["server.address"] == server_address
        if message_count > 1:
            assert span.attributes["messaging.batch.message_count"] == message_count

    def _verify_receive(self, *, span, dest, server_address, message_count):
        assert span.name == "ServiceBus.receive"
        assert span.kind == SpanKind.CLIENT
        self._verify_span_attributes(span=span)
        assert span.attributes["az.namespace"] == "Microsoft.ServiceBus"
        assert span.attributes["messaging.system"] == "servicebus"
        assert span.attributes["messaging.destination.name"] == dest
        assert span.attributes["messaging.operation"] == "receive"
        assert span.attributes["server.address"] == server_address
        for link in span.links:
            assert "enqueuedTime" in link.attributes
        if message_count > 1:
            assert span.attributes["messaging.batch.message_count"] == message_count

    def _verify_complete(self, *, span, dest, server_address):
        assert span.name == "ServiceBus.complete"
        assert span.kind == SpanKind.CLIENT
        self._verify_span_attributes(span=span)
        assert span.attributes["az.namespace"] == "Microsoft.ServiceBus"
        assert span.attributes["messaging.system"] == "servicebus"
        assert span.attributes["messaging.operation"] == "settle"
        assert span.attributes["server.address"] == server_address
        assert span.attributes["messaging.destination.name"] == dest

    @pytest.mark.live_test_only
    def test_servicebus_client_tracing_queue(self, config, tracing_helper):
        connection_string = config["servicebus_connection_string"]
        queue_name = config["servicebus_queue_name"]
        client = ServiceBusClient.from_connection_string(connection_string)

        with tracing_helper.tracer.start_as_current_span(name="root"):
            with client.get_queue_sender(queue_name) as sender:

                # Sending a single message
                sender.send_messages(ServiceBusMessage("Test foo message"))

                # Sending a batch of messages
                message_batch = sender.create_message_batch()
                message_batch.add_message(ServiceBusMessage("First batch foo message"))
                message_batch.add_message(ServiceBusMessage("Second batch foo message"))
                sender.send_messages(message_batch)

            send_spans = tracing_helper.exporter.get_finished_spans()
            server_address = sender.fully_qualified_namespace

            # We expect 5 spans to have finished: 2 send spans, and 3 message spans.
            assert len(send_spans) == 5

            # Verify the spans from the first send.
            self._verify_message(span=send_spans[0], dest=queue_name, server_address=server_address)
            self._verify_send(span=send_spans[1], dest=queue_name, server_address=server_address, message_count=1)

            # Verify span links from single send.
            link = send_spans[1].links[0]
            assert link.context.span_id == send_spans[0].context.span_id
            assert link.context.trace_id == send_spans[0].context.trace_id

            # Verify the spans from the second send.
            self._verify_message(span=send_spans[2], dest=queue_name, server_address=server_address)
            self._verify_message(span=send_spans[3], dest=queue_name, server_address=server_address)
            self._verify_send(span=send_spans[4], dest=queue_name, server_address=server_address, message_count=2)

            # Verify span links from batch send.
            assert len(send_spans[4].links) == 2
            link = send_spans[4].links[0]
            assert link.context.span_id == send_spans[2].context.span_id
            assert link.context.trace_id == send_spans[2].context.trace_id

            link = send_spans[4].links[1]
            assert link.context.span_id == send_spans[3].context.span_id
            assert link.context.trace_id == send_spans[3].context.trace_id

            tracing_helper.exporter.clear()

            # Receive all the sent spans.
            receiver = client.get_queue_receiver(queue_name=queue_name)
            with receiver:
                received_msgs = receiver.receive_messages(max_message_count=3, max_wait_time=10)
                for msg in received_msgs:
                    assert "foo" in str(msg)
                    receiver.complete_message(msg)

            receive_spans = tracing_helper.exporter.get_finished_spans()

            # We expect 4 spans to have finished: 1 receive span, and 3 settlement spans.
            assert len(receive_spans) == 4
            self._verify_receive(span=receive_spans[0], dest=queue_name, server_address=server_address, message_count=3)

            # Verify span links from receive.
            assert len(receive_spans[0].links) == 3
            assert receive_spans[0].links[0].context.span_id == send_spans[0].context.span_id
            assert receive_spans[0].links[1].context.span_id == send_spans[2].context.span_id
            assert receive_spans[0].links[2].context.span_id == send_spans[3].context.span_id

            # Verify settlement spans.
            self._verify_complete(span=receive_spans[1], dest=queue_name, server_address=server_address)
            self._verify_complete(span=receive_spans[2], dest=queue_name, server_address=server_address)
            self._verify_complete(span=receive_spans[3], dest=queue_name, server_address=server_address)

    @pytest.mark.live_test_only
    def test_servicebus_client_tracing_topic(self, config, tracing_helper):
        connection_string = config["servicebus_connection_string"]
        topic_name = config["servicebus_topic_name"]
        subscription_name = config["servicebus_subscription_name"]
        client = ServiceBusClient.from_connection_string(connection_string)

        with tracing_helper.tracer.start_as_current_span(name="root"):
            with client.get_topic_sender(topic_name) as sender:

                # Sending a single message
                sender.send_messages(ServiceBusMessage("Test foo message"))

                # Sending a batch of messages
                message_batch = sender.create_message_batch()
                message_batch.add_message(ServiceBusMessage("First batch foo message"))
                message_batch.add_message(ServiceBusMessage("Second batch foo message"))
                sender.send_messages(message_batch)

            send_spans = tracing_helper.exporter.get_finished_spans()
            server_address = sender.fully_qualified_namespace

            # We expect 5 spans to have finished: 2 send spans, and 3 message spans.
            assert len(send_spans) == 5

            # Verify the spans from the first send.
            self._verify_message(span=send_spans[0], dest=topic_name, server_address=server_address)
            self._verify_send(span=send_spans[1], dest=topic_name, server_address=server_address, message_count=1)

            # Verify span links from single send.
            link = send_spans[1].links[0]
            assert link.context.span_id == send_spans[0].context.span_id
            assert link.context.trace_id == send_spans[0].context.trace_id

            # Verify the spans from the second send.
            self._verify_message(span=send_spans[2], dest=topic_name, server_address=server_address)
            self._verify_message(span=send_spans[3], dest=topic_name, server_address=server_address)
            self._verify_send(span=send_spans[4], dest=topic_name, server_address=server_address, message_count=2)

            # Verify span links from batch send.
            assert len(send_spans[4].links) == 2
            link = send_spans[4].links[0]
            assert link.context.span_id == send_spans[2].context.span_id
            assert link.context.trace_id == send_spans[2].context.trace_id

            link = send_spans[4].links[1]
            assert link.context.span_id == send_spans[3].context.span_id
            assert link.context.trace_id == send_spans[3].context.trace_id

            tracing_helper.exporter.clear()

            # Receive all the sent spans.
            receiver = client.get_subscription_receiver(topic_name, subscription_name)
            with receiver:
                received_msgs = receiver.receive_messages(max_message_count=3, max_wait_time=10)
                for msg in received_msgs:
                    assert "foo" in str(msg)
                    receiver.complete_message(msg)

            receive_spans = tracing_helper.exporter.get_finished_spans()

            # We expect 4 spans to have finished: 1 receive span, and 3 settlement spans.
            assert len(receive_spans) == 4
            self._verify_receive(span=receive_spans[0], dest=topic_name, server_address=server_address, message_count=3)

            assert len(receive_spans[0].links) == 3
            assert receive_spans[0].links[0].context.span_id == send_spans[0].context.span_id
            assert receive_spans[0].links[1].context.span_id == send_spans[2].context.span_id
            assert receive_spans[0].links[2].context.span_id == send_spans[3].context.span_id

            # Verify settlement spans.
            self._verify_complete(span=receive_spans[1], dest=topic_name, server_address=server_address)
            self._verify_complete(span=receive_spans[2], dest=topic_name, server_address=server_address)
            self._verify_complete(span=receive_spans[3], dest=topic_name, server_address=server_address)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.compute.aio import ComputeManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestComputeManagementAvailabilitySetsOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ComputeManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_create_or_update(self, resource_group):
        response = await self.client.availability_sets.create_or_update(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            parameters={
                "location": "str",
                "id": "str",
                "name": "str",
                "platformFaultDomainCount": 0,
                "platformUpdateDomainCount": 0,
                "proximityPlacementGroup": {"id": "str"},
                "scheduledEventsPolicy": {
                    "scheduledEventsAdditionalPublishingTargets": {"eventGridAndResourceGraph": {"enable": bool}},
                    "userInitiatedReboot": {"automaticallyApprove": bool},
                    "userInitiatedRedeploy": {"automaticallyApprove": bool},
                },
                "sku": {"capacity": 0, "name": "str", "tier": "str"},
                "statuses": [
                    {
                        "code": "str",
                        "displayStatus": "str",
                        "level": "str",
                        "message": "str",
                        "time": "2020-02-20 00:00:00",
                    }
                ],
                "tags": {"str": "str"},
                "type": "str",
                "virtualMachineScaleSetMigrationInfo": {
                    "defaultVirtualMachineScaleSetInfo": {
                        "constrainedMaximumCapacity": bool,
                        "defaultVirtualMachineScaleSet": {"id": "str"},
                    },
                    "migrateToVirtualMachineScaleSet": {"id": "str"},
                },
                "virtualMachines": [{"id": "str"}],
            },
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_update(self, resource_group):
        response = await self.client.availability_sets.update(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            parameters={
                "platformFaultDomainCount": 0,
                "platformUpdateDomainCount": 0,
                "proximityPlacementGroup": {"id": "str"},
                "scheduledEventsPolicy": {
                    "scheduledEventsAdditionalPublishingTargets": {"eventGridAndResourceGraph": {"enable": bool}},
                    "userInitiatedReboot": {"automaticallyApprove": bool},
                    "userInitiatedRedeploy": {"automaticallyApprove": bool},
                },
                "sku": {"capacity": 0, "name": "str", "tier": "str"},
                "statuses": [
                    {
                        "code": "str",
                        "displayStatus": "str",
                        "level": "str",
                        "message": "str",
                        "time": "2020-02-20 00:00:00",
                    }
                ],
                "tags": {"str": "str"},
                "virtualMachineScaleSetMigrationInfo": {
                    "defaultVirtualMachineScaleSetInfo": {
                        "constrainedMaximumCapacity": bool,
                        "defaultVirtualMachineScaleSet": {"id": "str"},
                    },
                    "migrateToVirtualMachineScaleSet": {"id": "str"},
                },
                "virtualMachines": [{"id": "str"}],
            },
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_delete(self, resource_group):
        response = await self.client.availability_sets.delete(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_get(self, resource_group):
        response = await self.client.availability_sets.get(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_list_by_subscription(self, resource_group):
        response = self.client.availability_sets.list_by_subscription(
            api_version="2024-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_list(self, resource_group):
        response = self.client.availability_sets.list(
            resource_group_name=resource_group.name,
            api_version="2024-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_list_available_sizes(self, resource_group):
        response = self.client.availability_sets.list_available_sizes(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            api_version="2024-11-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_start_migration_to_virtual_machine_scale_set(self, resource_group):
        response = await self.client.availability_sets.start_migration_to_virtual_machine_scale_set(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            parameters={"virtualMachineScaleSetFlexible": {"id": "str"}},
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_cancel_migration_to_virtual_machine_scale_set(self, resource_group):
        response = await self.client.availability_sets.cancel_migration_to_virtual_machine_scale_set(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_validate_migration_to_virtual_machine_scale_set(self, resource_group):
        response = await self.client.availability_sets.validate_migration_to_virtual_machine_scale_set(
            resource_group_name=resource_group.name,
            availability_set_name="str",
            parameters={"virtualMachineScaleSetFlexible": {"id": "str"}},
            api_version="2024-11-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_availability_sets_begin_convert_to_virtual_machine_scale_set(self, resource_group):
        response = await (
            await self.client.availability_sets.begin_convert_to_virtual_machine_scale_set(
                resource_group_name=resource_group.name,
                availability_set_name="str",
                api_version="2024-11-01",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appplatform import AppPlatformManagementClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appplatform
# USAGE
    python deployments_create_or_update_custom_container.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = AppPlatformManagementClient(
        credential=DefaultAzureCredential(),
        subscription_id="00000000-0000-0000-0000-000000000000",
    )

    response = client.deployments.begin_create_or_update(
        resource_group_name="myResourceGroup",
        service_name="myservice",
        app_name="myapp",
        deployment_name="mydeployment",
        deployment_resource={
            "properties": {
                "deploymentSettings": {
                    "environmentVariables": {"env": "test"},
                    "livenessProbe": {
                        "disableProbe": False,
                        "failureThreshold": 3,
                        "initialDelaySeconds": 30,
                        "periodSeconds": 10,
                        "probeAction": {"path": "/health", "scheme": "HTTP", "type": "HTTPGetAction"},
                    },
                    "readinessProbe": {
                        "disableProbe": False,
                        "failureThreshold": 3,
                        "initialDelaySeconds": 30,
                        "periodSeconds": 10,
                        "probeAction": {"path": "/health", "scheme": "HTTP", "type": "HTTPGetAction"},
                    },
                    "resourceRequests": {"cpu": "1000m", "memory": "3Gi"},
                    "startupProbe": None,
                    "terminationGracePeriodSeconds": 30,
                },
                "instances": None,
                "source": {
                    "customContainer": {
                        "args": ["-c", "while true; do echo hello; sleep 10;done"],
                        "command": ["/bin/sh"],
                        "containerImage": "myContainerImage:v1",
                        "imageRegistryCredential": {"password": "myPassword", "username": "myUsername"},
                        "languageFramework": "springboot",
                        "server": "myacr.azurecr.io",
                    },
                    "type": "Container",
                },
            }
        },
    ).result()
    print(response)


# x-ms-original-file: specification/appplatform/resource-manager/Microsoft.AppPlatform/stable/2023-12-01/examples/Deployments_CreateOrUpdate_CustomContainer.json
if __name__ == "__main__":
    main()



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# --------------------------------------------------------------------------
from azure.core.pipeline.transport import RequestsTransport
from utils import HTTP_REQUESTS
import pytest

"""This file does a simple call to the testserver to make sure we can use the testserver"""


@pytest.mark.parametrize("http_request", HTTP_REQUESTS)
def test_smoke(port, http_request):
    request = http_request(method="GET", url="http://localhost:{}/basic/string".format(port))
    with RequestsTransport() as sender:
        response = sender.send(request)
        response.raise_for_status()
        assert response.text() == "Hello, world!"



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._database_migrations_sql_db_operations import DatabaseMigrationsSqlDbOperations
from ._database_migrations_sql_mi_operations import DatabaseMigrationsSqlMiOperations
from ._database_migrations_sql_vm_operations import DatabaseMigrationsSqlVmOperations
from ._operations import Operations
from ._sql_migration_services_operations import SqlMigrationServicesOperations
from ._resource_skus_operations import ResourceSkusOperations
from ._services_operations import ServicesOperations
from ._tasks_operations import TasksOperations
from ._service_tasks_operations import ServiceTasksOperations
from ._projects_operations import ProjectsOperations
from ._usages_operations import UsagesOperations
from ._files_operations import FilesOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "DatabaseMigrationsSqlDbOperations",
    "DatabaseMigrationsSqlMiOperations",
    "DatabaseMigrationsSqlVmOperations",
    "Operations",
    "SqlMigrationServicesOperations",
    "ResourceSkusOperations",
    "ServicesOperations",
    "TasksOperations",
    "ServiceTasksOperations",
    "ProjectsOperations",
    "UsagesOperations",
    "FilesOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Extensible enum. Indicates the action type. "Internal" refers to actions that are for internal
    only APIs.
    """

    INTERNAL = "Internal"
    """Actions are for internal-only APIs."""


class DeadlineType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The types of deadlines supported by ScheduledActions."""

    UNKNOWN = "Unknown"
    """Default value of Unknown."""
    INITIATE_AT = "InitiateAt"
    """Initiate the operation at the given deadline."""
    COMPLETE_BY = "CompleteBy"
    """Complete the operation by the given deadline."""


class OperationState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Values that define the states of operations in Scheduled Actions."""

    UNKNOWN = "Unknown"
    """The default value for the operation state enum"""
    PENDING_SCHEDULING = "PendingScheduling"
    """Operations that are pending scheduling"""
    SCHEDULED = "Scheduled"
    """Operations that have been scheduled"""
    PENDING_EXECUTION = "PendingExecution"
    """Operations that are waiting to be executed"""
    EXECUTING = "Executing"
    """Operations that are in the process of being executed"""
    SUCCEEDED = "Succeeded"
    """Operations that suceeded"""
    FAILED = "Failed"
    """Operations that have failed"""
    CANCELLED = "Cancelled"
    """Operations that have been Cancelled by the user"""
    BLOCKED = "Blocked"
    """Operations that are blocked"""


class OptimizationPreference(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The preferences customers can select to optimize their requests to ScheduledActions."""

    COST = "Cost"
    """Optimize while considering cost savings"""
    AVAILABILITY = "Availability"
    """Optimize while considering availability of resources"""
    COST_AVAILABILITY_BALANCED = "CostAvailabilityBalanced"
    """Optimize while considering a balance of cost and availability"""


class Origin(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit
    logs UX. Default value is "user,system".
    """

    USER = "user"
    """Indicates the operation is initiated by a user."""
    SYSTEM = "system"
    """Indicates the operation is initiated by a system."""
    USER_SYSTEM = "user,system"
    """Indicates the operation is initiated by a user or system."""


class ResourceOperationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of operation types that can be performed on resources using ScheduledActions."""

    UNKNOWN = "Unknown"
    """The default value for this enum type"""
    START = "Start"
    """Start operations on the resources"""
    DEALLOCATE = "Deallocate"
    """Deallocate operations on the resources"""
    HIBERNATE = "Hibernate"
    """Hibernate operations on the resources"""



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
DESCRIPTION:
    Asynchronous variant of sample_agents_basics_thread_and_run.py.
    It creates an agent, starts a new thread, and immediately runs it
    using the async Azure AI Agents client.

USAGE:
    python sample_agents_basics_thread_and_run_async.py

    Before running:

        pip install azure-ai-agents azure-identity aiohttp

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Azure AI Foundry project.
"""
import asyncio
import os

from azure.ai.agents.aio import AgentsClient
from azure.ai.agents.models import (
    AgentThreadCreationOptions,
    ThreadMessageOptions,
    MessageTextContent,
    ListSortOrder,
)
from azure.identity.aio import DefaultAzureCredential


async def main() -> None:
    async with DefaultAzureCredential() as credential:
        agents_client = AgentsClient(
            endpoint=os.environ["PROJECT_ENDPOINT"],
            credential=credential,
        )

        async with agents_client:
            agent = await agents_client.create_agent(
                model=os.environ["MODEL_DEPLOYMENT_NAME"],
                name="sample-agent",
                instructions="You are a helpful assistant that tells jokes.",
            )
            print(f"Created agent, agent ID: {agent.id}")

            # Prepare the initial user message
            initial_message = ThreadMessageOptions(
                role="user",
                content="Hello! Can you tell me a joke?",
            )

            # Create a new thread and immediately start a run on it
            run = await agents_client.create_thread_and_run(
                agent_id=agent.id,
                thread=AgentThreadCreationOptions(messages=[initial_message]),
            )

            # Poll the run as long as run status is queued or in progress
            while run.status in {"queued", "in_progress", "requires_action"}:
                await asyncio.sleep(1)
                run = await agents_client.runs.get(thread_id=run.thread_id, run_id=run.id)
                print(f"Run status: {run.status}")

            if run.status == "failed":
                print(f"Run error: {run.last_error}")

            # List all messages in the thread, in ascending order of creation
            messages = agents_client.messages.list(
                thread_id=run.thread_id,
                order=ListSortOrder.ASCENDING,
            )
            async for msg in messages:
                last_part = msg.content[-1]
                if isinstance(last_part, MessageTextContent):
                    print(f"{msg.role}: {last_part.text.value}")

            await agents_client.delete_agent(agent.id)
            print(f"Deleted agent {agent.id!r}")


if __name__ == "__main__":
    asyncio.run(main())



# pylint: disable=too-many-lines,too-many-statements
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import sys
from typing import Any, Callable, Dict, Optional, Type, TypeVar, Union, cast

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.async_arm_polling import AsyncARMPolling

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._export_jobs_operations import build_trigger_request

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore  # pylint: disable=ungrouped-imports
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class ExportJobsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.dataprotection.aio.DataProtectionMgmtClient`'s
        :attr:`export_jobs` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    async def _trigger_initial(  # pylint: disable=inconsistent-return-statements
        self, resource_group_name: str, vault_name: str, **kwargs: Any
    ) -> None:
        error_map: MutableMapping[int, Type[HttpResponseError]] = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_trigger_request(
            resource_group_name=resource_group_name,
            vault_name=vault_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request = _convert_request(_request)
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers["Location"] = self._deserialize("str", response.headers.get("Location"))
            response_headers["Retry-After"] = self._deserialize("int", response.headers.get("Retry-After"))

        if cls:
            return cls(pipeline_response, None, response_headers)  # type: ignore

    @distributed_trace_async
    async def begin_trigger(self, resource_group_name: str, vault_name: str, **kwargs: Any) -> AsyncLROPoller[None]:
        """Triggers export of jobs and returns an OperationID to track.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param vault_name: The name of the backup vault. Required.
        :type vault_name: str
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._trigger_initial(  # type: ignore
                resource_group_name=resource_group_name,
                vault_name=vault_name,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: AsyncPollingMethod = cast(
                AsyncPollingMethod, AsyncARMPolling(lro_delay, lro_options={"final-state-via": "location"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from typing import Any, TYPE_CHECKING, Union

from azure.core.credentials import AzureKeyCredential
from azure.core.pipeline import policies

from ._version import VERSION

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential


class ContentSafetyClientConfiguration:  # pylint: disable=too-many-instance-attributes,name-too-long
    """Configuration for ContentSafetyClient.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param endpoint: Supported Cognitive Services endpoints (protocol and hostname, for example:
     https://:code:`<resource-name>`.cognitiveservices.azure.com). Required.
    :type endpoint: str
    :param credential: Credential needed for the client to connect to Azure. Is either a
     AzureKeyCredential type or a TokenCredential type. Required.
    :type credential: ~azure.core.credentials.AzureKeyCredential or
     ~azure.core.credentials.TokenCredential
    :keyword api_version: The API version to use for this operation. Default value is "2023-10-01".
     Note that overriding this default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(self, endpoint: str, credential: Union[AzureKeyCredential, "TokenCredential"], **kwargs: Any) -> None:
        api_version: str = kwargs.pop("api_version", "2023-10-01")

        if endpoint is None:
            raise ValueError("Parameter 'endpoint' must not be None.")
        if credential is None:
            raise ValueError("Parameter 'credential' must not be None.")

        self.endpoint = endpoint
        self.credential = credential
        self.api_version = api_version
        self.credential_scopes = kwargs.pop("credential_scopes", ["https://cognitiveservices.azure.com/.default"])
        kwargs.setdefault("sdk_moniker", "ai-contentsafety/{}".format(VERSION))
        self.polling_interval = kwargs.get("polling_interval", 30)
        self._configure(**kwargs)

    def _infer_policy(self, **kwargs):
        if isinstance(self.credential, AzureKeyCredential):
            return policies.AzureKeyCredentialPolicy(self.credential, "Ocp-Apim-Subscription-Key", **kwargs)
        if hasattr(self.credential, "get_token"):
            return policies.BearerTokenCredentialPolicy(self.credential, *self.credential_scopes, **kwargs)
        raise TypeError(f"Unsupported credential: {self.credential}")

    def _configure(self, **kwargs: Any) -> None:
        self.user_agent_policy = kwargs.get("user_agent_policy") or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get("headers_policy") or policies.HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get("proxy_policy") or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get("logging_policy") or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get("http_logging_policy") or policies.HttpLoggingPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get("custom_hook_policy") or policies.CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get("redirect_policy") or policies.RedirectPolicy(**kwargs)
        self.retry_policy = kwargs.get("retry_policy") or policies.RetryPolicy(**kwargs)
        self.authentication_policy = kwargs.get("authentication_policy")
        if self.credential and not self.authentication_policy:
            self.authentication_policy = self._infer_policy(**kwargs)


class BlocklistClientConfiguration:  # pylint: disable=too-many-instance-attributes,name-too-long
    """Configuration for BlocklistClient.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param endpoint: Supported Cognitive Services endpoints (protocol and hostname, for example:
     https://:code:`<resource-name>`.cognitiveservices.azure.com). Required.
    :type endpoint: str
    :param credential: Credential needed for the client to connect to Azure. Is either a
     AzureKeyCredential type or a TokenCredential type. Required.
    :type credential: ~azure.core.credentials.AzureKeyCredential or
     ~azure.core.credentials.TokenCredential
    :keyword api_version: The API version to use for this operation. Default value is "2023-10-01".
     Note that overriding this default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(self, endpoint: str, credential: Union[AzureKeyCredential, "TokenCredential"], **kwargs: Any) -> None:
        api_version: str = kwargs.pop("api_version", "2023-10-01")

        if endpoint is None:
            raise ValueError("Parameter 'endpoint' must not be None.")
        if credential is None:
            raise ValueError("Parameter 'credential' must not be None.")

        self.endpoint = endpoint
        self.credential = credential
        self.api_version = api_version
        self.credential_scopes = kwargs.pop("credential_scopes", ["https://cognitiveservices.azure.com/.default"])
        kwargs.setdefault("sdk_moniker", "ai-contentsafety/{}".format(VERSION))
        self.polling_interval = kwargs.get("polling_interval", 30)
        self._configure(**kwargs)

    def _infer_policy(self, **kwargs):
        if isinstance(self.credential, AzureKeyCredential):
            return policies.AzureKeyCredentialPolicy(self.credential, "Ocp-Apim-Subscription-Key", **kwargs)
        if hasattr(self.credential, "get_token"):
            return policies.BearerTokenCredentialPolicy(self.credential, *self.credential_scopes, **kwargs)
        raise TypeError(f"Unsupported credential: {self.credential}")

    def _configure(self, **kwargs: Any) -> None:
        self.user_agent_policy = kwargs.get("user_agent_policy") or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get("headers_policy") or policies.HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get("proxy_policy") or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get("logging_policy") or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get("http_logging_policy") or policies.HttpLoggingPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get("custom_hook_policy") or policies.CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get("redirect_policy") or policies.RedirectPolicy(**kwargs)
        self.retry_policy = kwargs.get("retry_policy") or policies.RetryPolicy(**kwargs)
        self.authentication_policy = kwargs.get("authentication_policy")
        if self.credential and not self.authentication_policy:
            self.authentication_policy = self._infer_policy(**kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------

from .trigger import Trigger


class FileEventTrigger(Trigger):
    """Trigger details.

    Variables are only populated by the server, and will be ignored when
    sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar id: The path ID that uniquely identifies the object.
    :vartype id: str
    :ivar name: The object name.
    :vartype name: str
    :ivar type: The hierarchical type of the object.
    :vartype type: str
    :param kind: Required. Constant filled by server.
    :type kind: str
    :param source_info: Required. File event source details.
    :type source_info: ~azure.mgmt.edgegateway.models.FileSourceInfo
    :param sink_info: Required. Role sink info.
    :type sink_info: ~azure.mgmt.edgegateway.models.RoleSinkInfo
    :param custom_context_tag: A custom context tag typically used to
     correlate the trigger against its usage. For example, if a periodic timer
     trigger is intended for certain specific IoT modules in the device, the
     tag can be the name or the image URL of the module.
    :type custom_context_tag: str
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'kind': {'required': True},
        'source_info': {'required': True},
        'sink_info': {'required': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'kind': {'key': 'kind', 'type': 'str'},
        'source_info': {'key': 'properties.sourceInfo', 'type': 'FileSourceInfo'},
        'sink_info': {'key': 'properties.sinkInfo', 'type': 'RoleSinkInfo'},
        'custom_context_tag': {'key': 'properties.customContextTag', 'type': 'str'},
    }

    def __init__(self, **kwargs):
        super(FileEventTrigger, self).__init__(**kwargs)
        self.source_info = kwargs.get('source_info', None)
        self.sink_info = kwargs.get('sink_info', None)
        self.custom_context_tag = kwargs.get('custom_context_tag', None)
        self.kind = 'FileEvent'



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
import time
from abc import abstractmethod
from retry import retry
import warnings

from devtools_testutils import AzureRecordedTestCase

from _shared.utils import get_http_logging_policy
from azure.communication.jobrouter import (
    JobRouterClient,
    JobRouterAdministrationClient,
)
from azure.communication.jobrouter.models import (
    RouterJobStatus,
    CancelJobOptions,
    CompleteJobOptions,
    CloseJobOptions,
)


class RouterRecordedTestCase(AzureRecordedTestCase):
    @abstractmethod
    def clean_up(self):
        pass

    def create_client(self) -> JobRouterClient:
        return JobRouterClient.from_connection_string(
            conn_str=self.connection_string, http_logging_policy=get_http_logging_policy()
        )

    def create_admin_client(self) -> JobRouterAdministrationClient:
        return JobRouterAdministrationClient.from_connection_string(
            conn_str=self.connection_string, http_logging_policy=get_http_logging_policy()
        )

    @retry(Exception, delay=3, tries=3)
    def clean_up_job(self, job_id, **kwargs):
        router_client: JobRouterClient = self.create_client()
        router_job = router_client.get_job(job_id=job_id)
        suppress_errors = kwargs.pop("suppress_errors", False)

        try:
            if router_job.status == RouterJobStatus.PENDING_CLASSIFICATION:
                # cancel and delete job
                router_client.cancel_job(job_id, CancelJobOptions(disposition_code="JobCancelledAsPartOfTestCleanUp"))
                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.QUEUED:
                # cancel and delete job
                router_client.cancel_job(job_id, CancelJobOptions(disposition_code="JobCancelledAsPartOfTestCleanUp"))
                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.ASSIGNED:
                # complete, close and delete job
                worker_assignments = router_job.assignments

                for assignment_id, job_assignment in worker_assignments.items():
                    router_client.complete_job(job_id, assignment_id)
                    router_client.close_job(job_id, assignment_id)

                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.COMPLETED:
                # close and delete job
                worker_assignments = router_job.assignments

                for assignment_id, job_assignment in worker_assignments.items():
                    router_client.close_job(job_id, assignment_id)

                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.CLOSED:
                # delete job
                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.CANCELLED:
                # delete job
                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.CLASSIFICATION_FAILED:
                # delete job
                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.CREATED:
                # cancel and delete job
                router_client.cancel_job(job_id, CancelJobOptions(disposition_code="JobCancelledAsPartOfTestCleanUp"))
                router_client.delete_job(job_id=job_id)
            elif router_job.status == RouterJobStatus.WAITING_FOR_ACTIVATION:
                # cancel and delete job
                router_client.cancel_job(job_id, CancelJobOptions(disposition_code="JobCancelledAsPartOfTestCleanUp"))
                router_client.delete_job(job_id=job_id)
            else:
                pass
        except Exception as e:
            msg = f"Deletion of job failed: {job_id}"
            warnings.warn(UserWarning(msg))
            print(e)
            if not suppress_errors:
                raise e

    def _poll_until_no_exception(self, fn, expected_exception, *args, **kwargs):
        """polling helper for live tests because some operations take an unpredictable amount of time to complete"""
        max_retries = kwargs.pop("max_retries", 20)
        retry_delay = kwargs.pop("retry_delay", 3)

        for i in range(max_retries):
            try:
                return fn(*args, **kwargs)
            except expected_exception:
                if i == max_retries - 1:
                    raise
                if self.is_live:
                    time.sleep(retry_delay)

    def _poll_until_exception(self, fn, expected_exception, max_retries=20, retry_delay=3):
        """polling helper for live tests because some operations take an unpredictable amount of time to complete"""

        for _ in range(max_retries):
            try:
                fn()
                if self.is_live:
                    time.sleep(retry_delay)
            except expected_exception:
                return

        raise AssertionError("expected exception {expected_exception} was not raised")



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import argparse
import os
import sys
import logging
from os import path

# import common_task module
root_dir = path.abspath(path.join(path.abspath(__file__), "..", "..", ".."))
common_task_path = path.abspath(path.join(root_dir, "scripts", "devops_tasks"))
sys.path.append(common_task_path)
from common_tasks import get_installed_packages

def verify_packages(package_file_path):
    # this method verifies packages installed on machine is matching the expected package version
    # packages.txt file expects to have list of packages and version in format <package-name>==<version>

    packages = []
    with open(package_file_path, "r") as packages_file:
        packages = packages_file.readlines()
    packages = [p.replace('\n', '') for p in packages]

    # packages.txt is created by package installation script. But this script can be reused to verify installed packages
    # Add a sanity check to ensure content in file is in expected format
    invalid_lines = [p for p in packages if '==' not in p]
    if invalid_lines:
        logging.error("packages.txt has package details in invalid format. Expected format is <package-name>==<version>")
        sys.exit(1)

    # find installed and expected packages
    installed = {}
    for p in get_installed_packages():
        if "==" in p:
            [package, version] = p.split("==")
            installed[package.upper()] = version
    expected = {}
    for p in packages:
        [package, version] = p.split("==")
        expected[package.upper()] = version

    missing_packages = [pkg for pkg in expected.keys() if installed.get(pkg) != expected.get(pkg)]

    if missing_packages:
        logging.error("Version is incorrect for following package[s]")
        for package in missing_packages:
            logging.error("%s, Expected[%s], Installed[%s]", package, expected[package], installed[package])
        sys.exit(1)
    else:
        logging.info("Verified package version")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Install either latest or minimum version of dependent packages."
    )

    parser.add_argument(
        "-f",
        "--packages-file",
        dest="packages_file",
        help="Path to a file that has list of packages and version to verify",
        required=True,
    )

    args = parser.parse_args()
    if os.path.exists(args.packages_file):
        verify_packages(args.packages_file)
